#/* Copyright 1998-2011 Research Foundation State University of New York */
#
#/* This file is part of QuB.                                            */
#
#/* QuB is free software; you can redistribute it and/or modify          */
#/* it under the terms of the GNU General Public License as published by */
#/* the Free Software Foundation, either version 3 of the License, or    */
#/* (at your option) any later version.                                  */
#
#/* QuB is distributed in the hope that it will be useful,               */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
#/* GNU General Public License for more details.                         */
#
#/* You should have received a copy of the GNU General Public License,   */
#/* named LICENSE.txt, in the QuB program directory.  If not, see        */
#/* <http://www.gnu.org/licenses/>.                                      */

import os
import os.path
import re
import qubtree
import weakref

# Sep 2005 Chris
#
# Preferences management
# Centralizes the disk access.
# To play along:
#     maintain your current prefs in a QUB_NODE
#     come up with a category Name
#     init: update my prefs from SettingsMgr[Name].Active
#           SettingsMgr[Name].active := my prefs tree
#           SettingsMgr[Name].onSet := f(settings-cat, updates-tree) that updates my prefs tree
#     don't forget to unset OnSet before you are destroyed
#     qub will take care of saving and restoring Active across sessions
#     for multiple documents sharing a category:
#           only the front one should be active
#           (owning active and onSet)
#           the init procedure above will give new docs the same settings as the last viewed doc
#     when showing doc properties, switch active and onset over to the dialog
# app should start by calling InitSettings(central-settings-dir, user-settings-dir)

# todo:    the gui components provided handle loading,saving,switching

def printme(s):
    print s
    return s

class Settings(object):
    def __init__(self, mgr, name):
        self.name = name
        self.active = mgr.load(name, '__active')
        if self.active.isNull:
            self.active = qubtree.Node('Properties')
        self.hDefOnSet = self._defOnSet
        self._onSet = self.hDefOnSet
    def _defOnSet(self, settings, updates):
        pass # ?
    def _setOnSet(self, onSet):
        if onSet:
            self._onSet = weakref.ref(onSet) # , lambda l:printme("released onSet"))
        else:
            self._onSet = weakref.ref(self.hDefOnSet) # , lambda l:printme("released def onSet"))
    def _getOnSet(self):
        if not self._onSet():
            self.onSet = None
        return self._onSet()
    onSet = property(lambda s: s._getOnSet(), lambda s,x: s._setOnSet(x))
    def setProperties(self, updates):
        self.onSet(self, updates)
    def listNames(self):
        return SettingsMgr.listNames(self.name)
    def save(self, name):
        return SettingsMgr.save(self.active, self.name, name)
    def load(self, name):
        props = SettingsMgr.load(self.name, name)
        if not props.isNull:
            self.setProperties(props)
        return not props.isNull
    def exportFile(self, path):
        return self.active.clone().saveAs(path)
    def importFile(self, path):
        props = qubtree.Open(path, True)
        props.close()
        if not props.isNull:
            self.setProperties(props)
        return not props.isNull
    def saveActive(self):
        return SettingsMgr.save(self.active, self.name, '__active')

class SettingsManager(object):
    def __init__(self, gdir, udir):
        self.gdir = gdir # global search-path
        self.udir = udir # user   search-path (first)
        self.cats = {}
    def saveAll(self):
        for cat in self.cats.itervalues():
            cat.saveActive()
    def listNames(self, cat):
        names = {}
        for root in (self.gdir, self.udir):
            for base, dirs, files in os.walk(os.path.join(root,cat)):
                dirs[:] = [] # don't recurse
                for f in files:
                    match = re.match(r"(.+)\.qpr", f)
                    if match and (f[:2] != "__"):
                        names[match.group(1)] = 1
        return names.keys()
    def save(self, props, cat, name):
        path = os.path.join(self.udir, cat)
        try:
            mkdir_r(path)
            return props.clone().saveAs(os.path.join(path, name+'.qpr'))
        except:
            return False
    def load(self, cat, name):
        props = qubtree.Open(os.path.join(self.udir, cat, name+'.qpr'))
        if props.isNull:
            props = qubtree.Open(os.path.join(self.gdir, cat, name+'.qpr'))
        props.close()
        return props
    def __getitem__(self, cat):
        if not self.cats.has_key(cat):
            self.cats[cat] = Settings(self, cat)
        return self.cats[cat]

def mkdir_r(path):
    container = os.path.split(path)[0]
    if path == container:
        return
    if not os.path.exists(path):
        mkdir_r(container)
        os.mkdir(path)

SettingsMgr = None
def InitSettings(gdir, udir):
    global SettingsMgr
    SettingsMgr = SettingsManager(gdir, udir)

